
rawset(_G, "HugeQuest", "V1") -- Heyo!

rawset(_G, "growplus", {speedmod = FRACUNIT/10})

rawset(_G, "hq_tips", CV_RegisterVar({
	name = "hq_tips",
	defaultvalue = "On",
	possiblevalue = CV_OnOff
}))

local init = false
addHook("NetVars", function(net)
	init = net(init)
end)
addHook("ThinkFrame", function()
	if leveltime == 0 then return end
	if not init then
		if HOSTMOD then
			HM_Scoreboard_AddMod({disp = "Midair MT Storage", var = "hq_mtstore"})
			HM_Scoreboard_AddMod({disp = "Grow Timer Stack", var = "hq_huge"})
			HM_Scoreboard_AddMod({disp = "Grow Orbi Split", var = "hq_huge"})
			HM_Scoreboard_AddMod({disp = "Grow Banana Split", var = "hq_huge"})
			HM_Scoreboard_AddMod({disp = "Smarter Jawz", var = "hq_smartjawz"})
			HM_Scoreboard_AddMod({disp = "Inv Timer Stack", var = "hq_sparkle"})
			HM_Scoreboard_AddMod({disp = "Grow Orbi Gift", var = "hq_orbigift"})
		end
		init = true
	end
end)

local tips = {
	[[Grow+ allows for item usage while active. Go wreak havoc!]],
	[[Item odds during Grow+ heavily favor attack items, with a chance to roll 4x Orbis or 10x Bananas!]],
	[[Most attacks items are now twice as large when used during Grow+. Some even split on impact!]],
	[[Big Orbis fired during Grow+ will split into two smaller Orbis upon impact with an item, player, or wall. M-M-Multiball!]],
	[[Big Bananas dropped/shot during Grow+ will split into three smaller bananas upon impact with an item or a player. Praise be to Banan.]],
	[[If hq_offroad is enabled, Grow+ will offer offroad immunity, allowing you to gain ground faster at the cost of a smaller speed bonus and duration compared to Vanilla Grow. Offroad immunity is offered by a green flash while enabled.]],
	[[Your Grow+ timer will extend if using another Grow while active.]],
	[[If hq_sparkle is enabled, your Invincibility timer will extend if using another Invincibility while active.]],
	[[If hq_mtstore is enabled, if you have an active mini turbo while airborne, it will be preserved until you land.]],
	[[If hq_mtstore is enabled, if you have an active mini turbo during sneaker effects, it will be preserved until the effects wear off, and your fast lines will be green!]],
	[[If hq_smartjawz is enabled, Jawz will be more aggressive and warp to their targets if they're too far.]],
	[[If hq_smartjawz is enabled, Jawz targets will hear a distinct sound indicating it has warped nearby. Red means RUN.]],
	[[If hq_smartjawz is enabled, Jawz will chase players across jumps, and midair hits will cause the victim to plummet to their doom.]],
	[[If hq_smartjawz is enabled, Jawz can track their targets without line of sight. When you hear the lock-on sound, fire!]],
	[[To disable end-of-race tips for HugeQuest, see luafiles/hugequest.cfg.]],
}

local dispinfo = {
	{"Grow+", hq_huge},
	{"Grow+ Free Orbi", hq_orbigift},
	{"Grow+ Double Damage", hq_yeet},
	{"Grow+ Offroad Immunity", hq_offroad},
	{"Midair MT Storage", hq_mtstore},
	{"Stackable Inv. Duration", hq_sparkle},
	{"Quantum Jawz", hq_smartjawz}

}

COM_AddCommand("hq_info", function(p)

	CONS_Printf(p, "\x89".."Running HugeQuest "..HugeQuest)
	CONS_Printf(p, "Feature List:")
	for i = 1, #dispinfo
		local info = dispinfo[i]
		local clr = info[2].value and "\x8B".."ON" or "\x8A".."OFF"

		CONS_Printf(p, "\x89"..info[2].name.."\x80"..": ("..info[1]..") - "..clr)
	end
	CONS_Printf(p, "Type \'hq_help <command name>\' for more details about the above commands")
end)

local help = {
	{"Enables item usage during Grow and alters item properties while active. Using Grow while Grow+ is active extends the duration.", hq_huge},
	{"Grants a temporary Orbinaut on Grow+ activation. It will disappear if not used when Grow+ runs out.", hq_orbigift},
	{"Extends spinout inflicted when hitting players with items used by a Grow+ player.", hq_yeet},
	{"Grants offroad immunity (indicated by a green flash) during Grow+, at the cost of reduced duration (10 seconds) and total speed bonus (65%). If OFF, duration and bonus are equal to Vanilla Grow (12 seconds and 80% total speed bonus).", hq_offroad},
	{"Preserves Mini-Turbo in the air and during sneaker effects. MT storage during sneaker effects is indicated by green fastlines.", hq_mtstore},
	{"Extends Invincibility duration when used while Invincibility is active.", hq_sparkle},
	{"Jawz can warp to distant targets and follow players across jumps.", hq_smartjawz}
}

COM_AddCommand("hq_help", function(p, arg)

	if not arg return end
	arg = arg:lower()

	for i = 1, #help
		if arg == help[i][2].name
			CONS_Printf(p, "\x89"..help[i][2].name.."\x80"..":\n"..help[i][1])
			return
		end
	end
	CONS_Printf(p, "No command named \'"..arg.."\' in HugeQuest. Please use \'hq_info\' to view a list of all the commands.")
end)



addHook("ThinkFrame", function()
	if leveltime == 0 then return end
	if leveltime == 1 then
		for p in players.iterate
			p.hqtips = 0
		end
	end
	
	for p in players.iterate do
		if not (p.mo and p.mo.valid and (not p.spectator)) continue end
		if p.exiting or (p.lives == 0) then
			p.hqtips = $ or 0
			p.hqtips = $ + 1
			p.hqtipreceived = $ or {}
			if p.hqtips == 2*TICRATE then
				local pendingtip = (leveltime%#tips)
				local received = 0
				for key, val in pairs(p.hqtipreceived)
					received = $ + 1
				end
				if received < #tips then
					repeat
						pendingtip = ($+1)%#tips
					until not p.hqtipreceived[pendingtip]
					p.hqtipreceived[pendingtip] = true
					if hq_tips.value then
						chatprintf(p, "\x88*HugeQuest Tip: "..tips[pendingtip+1], true)
					end
				end
			end
		end
	end	
		
end)